package framework.config;

import java.io.*;
import java.util.*;

import javax.xml.ws.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Configuration utilities.
 */
public class ConfigUtil {

    //
    // Members
    //

    /** Logging */
    private static Log log = LogFactory.getLog(ConfigUtil.class);


    //
    // Utility methods
    //

    /**
     * Create new properties object from resource file.
     *
     * @param resourcePath resource file path (e.g. /file.properties)
     * @return properties object with loaded properties or null if resource not found
     */
    public static Properties getResourceAsProperties(String resourcePath) {
        Properties newProps = null;
        InputStream is = null;
        try {
            is = Config.class.getResourceAsStream(resourcePath);
            if(is == null) {
                log.debug("Resource " + resourcePath + " not found.");
                return null;
            }
            newProps = new Properties();
            newProps.load(is);
        } catch (IOException e) {
            log.debug("caught io exception when reading resource file " +
                      resourcePath + "; returning null", e);
            return null;
        } finally {
            try {
                if(is != null)
                    is.close();
            } catch (IOException e) {
                log.debug("caught io exception while closing InputStream; ignoring", e);
            }
        }
        return newProps;
    }

    /**
     * Copy all source properties to destination properties, one by one.
     */
    public static void copyProperties(Properties sourceProps, Properties destinationProps) {
        copyProperties(sourceProps, destinationProps, "", "");
    }

    /**
     * Copy all source properties to destination properties, one by one,
     * adding the specified prefix and suffix to the key name
     */
    public static void copyProperties(Properties sourceProps, Properties destinationProps,
                                      String propNamePrefix, String propNameSuffix) {
        // check arguments
        if(sourceProps == null)
            throw new IllegalArgumentException("Can't copy properties from null source!");
        if(destinationProps == null)
            throw new IllegalArgumentException("Can't copy properties to null destination!");
        if(propNamePrefix == null)
            propNamePrefix = "";
        if(propNameSuffix == null)
            propNameSuffix = "";

        // copy keys using prefix and suffix
        for(Object keyObject : sourceProps.keySet()) {
            String keyString = (String) keyObject;
            int size = propNamePrefix.length() + keyString.length() + propNameSuffix.length();
            if(size > keyString.length()) {
                // new key is different, build it
                StringBuilder sb = new StringBuilder(size);
                sb.append(propNamePrefix);
                sb.append(keyString);
                sb.append(propNameSuffix);
                keyString = sb.toString();
            }
            destinationProps.setProperty(keyString,(String)sourceProps.get((String)keyObject));
        }
    }

}
